#include <QtGui/QApplication>
#include "lqminiterm.h"
#include "ui_lqminiterm.h"
#include <ctype.h> // alnum()
#include <stdio.h>

void dbg(){}


static bool input_ready;
static char* input_str;
// static bool text_mode;

static void term_print(lqMiniTerm* term, const char* buf, bool isPrompt = false);

static void term_print(lqMiniTerm* term, const char* buf, bool isPrompt)
{
  term->append(buf, isPrompt);
}

static void prompt(lqMiniTerm* term)
{
  bool isPrompt;  
  term_print(term, "!@#$%%#@>", isPrompt = true);
}

static void test_interp(lqMiniTerm* term, const char* str)
{
  char buf[256];
  
  if(0 == strcmp(str, "ls"))
  {
    FILE* fp = popen("ls", "r");
    if(!fp)
      return;
    while(!feof(fp))
    {
      *buf = 0;
      fgets(buf, 256, fp);
      term->append(buf);
    }
    fclose(fp);
  }
  else if(0 == strcmp(str, "exit"))
  {
    term->append("See yuh later!\n");
    term->update();
    // I/O pause, to allow the OS's other threads to 
    // catch up, including X11.  1 ms should be adequate.
    usleep(1000);
    
    // now we want to keep control of the program flow 
    // here so we process events (so the text can show)
    // and then exit.
    qApp->processEvents();
    sleep(2.5);        
    term->close();
  }
    
  return;
}

// user programmable loop func in app-main()
// this just does shell commands.
extern "C" long loop_func(lqMiniTerm* term)
{
  long handled = 1; // for clarity below
  
  // avoid recursion
  if(term->closing)
    return handled;
  
  // only takes input in edit mode
  if(!term->edit_mode)
    return handled;
  
  // input_ready is set by onNewline in the handler
  if(!input_ready)  
    return handled;
  
  // Ready to go now...
  
  // Typically we'd run an interpreter with the text input
  // so far.
  
  test_interp(term, input_str);
  
  // no feedback to react to so we're done with this loop.
  
  // reset for next and print the prompt
  input_ready = false;  
  prompt(term);  
  
  // get more input from the editor
  term->edit_mode = true;
  
  // and we're done until next loop which is called by
  // lqMiniTerm::run()
  
  return handled;
}


#define UNUSED(x) x=x
extern "C" long handler(lqMiniTerm* term, uint msg, long wparam, void* lparam)
{
  UNUSED(lparam);
  
  int prompt_min = term->prompt_start;
  int prompt_max = term->prompt_end;
  int pos = term->position();

  switch(msg)
  {    
    case msg_init:
      term_print(term,
          "This is a simple interpreter that only does 'ls' in PWD\n"
          "as an example of possible usage.\n\n"
          "Other text will simply be ignored except for one more\n"
          "mystery command that you'll have to find in the sources ;-).\n\n");
      
      term->edit_mode = true; // use as input
      break;
      
    case msg_onKeyPress:
    {
      QTextCursor chk = term->editor()->textCursor();
      if(! term)
      {
        fprintf(stderr, "No term set in EventRelay class\n");
        abort();
      }
      
      QKeyEvent* e = (QKeyEvent*)lparam; // cast generic QEvent to QKeyEvent
                  
      switch(e->key())
      {
        case 0x1000014: // Qt::Key_Right: 
        {
          // skip over prompt area
          if(pos == (prompt_min - 1)) // move to end of prompt and eat key
          {            
            term->setCursorPosition(prompt_max);
            return true; // handled = eat key
          }
          break; // not handled
        }
        case 0x1000003:     // Qt::Key_Backspace:
        {
          // don't stray into prompt area
          if(pos == prompt_max)
            return true;    // handled = eat key
          break;            // not handled
        }
        case 0x1000012:     // Qt::Key_Left:
        {
          if(pos == prompt_max)
          {
            // skip back past prompt start to previous block
            /// might need to be -2 here, not sure how the lines are handled
            /// but we want to go up one block and to the end of the line
            term->setCursorPosition(prompt_min - 1);
            return true;
          }
          break;
        }

        case 0x1000010: // Qt::Key_Home:
        {
          if(pos >= prompt_min)
          {
            // Notes:
            // QPlainTextEdit(1,1,1,1,1);
            // QTextCursor(1,1,1,1,1);
            term->setCursorPosition(prompt_max);
            return true; // handled
          }
          break; // not handled
        }        
        case 0x1000005: // Qt::Key_Enter: (keypad)
        case 0x1000004: // Qt::Key_Return:
        {
          // done always in the lqtermedit KeyPressEvent dispatcher
          // term->editor()->moveCursor(QTextCursor::EndOfLine);
          break;
        }
        // other stuff to handle would be ENTER move if in read only
        // Ctrl-V move before paste if in read only
      }
      
      return false; // not handled
    }
    
    case msg_onClose:
      // printf("Handled: Closing\n");
      term->hide(); // signal to runner we can quit
      term->close();
      break;
    
    case msg_onNewline: // typical newline handler
    {
      if(!term->edit_mode)
        break;
      
      // This is redundant but harmless
      term->moveCursor(QTextCursor::EndOfLine);
      pos = term->position(); // now

      // Now we work with the loop func to get valid input
      const char* s = term->getInput();
      
      int prompt_len = prompt_max - prompt_min;
      int len = strlen(s) - prompt_len;
      
      // A test routine for the commandline to see what we've got
      if((len > 0) && !input_ready)
      {
        input_str = (char*)realloc(input_str, len+1);
        memcpy(input_str, s + prompt_len, len);
        input_str[len] = 0;
        printf("Input: %s\n", input_str);
      }

      // flag LoopFunc that it's ready
      input_ready = true; 
      break;
    }
    case msg_onCursorPositionChanged:
    {
      // if it's on the same line as the prompt limit it to prompt_len offset
      // else make it read-only
      int new_pos = (int)wparam;
      
      if(new_pos < term->prompt_end)
        term->setReadOnly(true);
      else
        term->setReadOnly(false);
    }
  }
  return true; // handled
}



extern void visit_lqTermEdit();  


int main(int argc, char *argv[])
{
  dbg();
  
  visit_lqTermEdit();  
  
  // this sets the global qApp instance
  new QApplication (argc, argv);
  
  lqMiniTerm term(loop_func, handler);
  
  term.show();
  
  // Reposition, resize etc. here after shown()
  
  prompt(&term);
  // instead of return qApp->exec();
  return term.run();
}

