// file: lqminiterm.cpp

#include <QDesktopWidget>
#include "lqminiterm.h"
#include "ui_lqminiterm.h"
#include <stdio.h>
/*!
  \brief The constructor
  The user supplies a LoopFunc and a HandlerFunc and optional parent pointer.
*/


lqMiniTerm::lqMiniTerm(LoopFunc loop_func, HandlerFunc handler_func, QWidget *parent) :
    QWidget(parent),
    ui(new Ui::lqMiniTerm)
{
    ui->setupUi(this);
    loopFunc = loop_func;
    handler = handler_func;
    closing = false;        // don't recurse
    edit_mode = true;    // not as output
    prompt_start = prompt_end = 0;
    init();
}

/*!
  \brief The destructor
  Additional cleanups should be done in the onClose() slot/callback
*/

lqMiniTerm::~lqMiniTerm()
{
    delete ui;
}

/*!
  \brief Calls the user defined handler callback with msg_onClose to do cleanups.
*/
    
void lqMiniTerm::onClose()
{
  closing = true;
  handler(this, msg_onClose, 0, 0);
}

/*! \brief calls the user defined handler callback with msg_onNewline
  The user must check edit_mode and handle or ignore those of interest.
*/

void lqMiniTerm::onNewline(int argA)
{
  handler(this, msg_onNewline, argA, 0);  
}

/*! \brief slot calls user defined handler callback for cursor movement with msg_onCursorPositionChanged
  This isn't working properly yet.  Well, it sort of is, in that it determines 
  if we should be in read/write or read-only mode when in certain areas on the 
  editor's screen.
*/

void lqMiniTerm::onCursorPositionChanged()
{
  QTextCursor cursor = ui->plainTextEdit->textCursor();
  int pos = cursor.position();  
  //printf("Doing %s\n", __func__);
  handler(this, msg_onCursorPositionChanged, pos, 0);
}

/*! \brief slot allows getting input from buttons, etc.
  \param 
 */

void lqMiniTerm::onExternalInput(QString argA)
{
  // if(edit_mode) // = true;
  //    miniterm->append(argA);
  const char* s = argA.toAscii();
  handler(this, msg_onExternalInput, 0, (void*)s);

}

/*! \brief calls user defined handler callback for with msg_init.
*/

void lqMiniTerm::init()
{
  handler(this, msg_init, 0, 0);
}

/*! \brief runs the window instead of 'exec()'.
  The run() function replaces exec().  It runs lqMiniterm::LoopFunc() each 
  cycle.  By default it stops and waits for events, to consume the least cpu
  time but if you set the timeout to any positive integer (or 0), the loop 
  will run every 'timeout' in milliseconds (plus the time consumded by 
  processEvents()).
*/

int lqMiniTerm::run(int timeout)
{
  for(;;)
  {
    if(timeout >= 0)
    {
      qApp->processEvents(); // no wait
      usleep(timeout * 1000);
    }
    else
      qApp->processEvents(QEventLoop::WaitForMoreEvents);
    
    if(!isVisible())
      break;
    
    loopFunc(this);
  }
  return 0;
}

/*! \brief returns the cursor position relative to the start of the document.
*/

int lqMiniTerm::position()
{
  return ui->plainTextEdit->textCursor().position();
}

/*! \brief sets the cursor position relative to the start of the document.
 */

void lqMiniTerm::setPosition(int pos)
{
  ui->plainTextEdit->textCursor().setPosition(pos);
}


/*! \brief sets the display to read-only mode or editable depending on b
*/

void lqMiniTerm::setReadOnly(bool b)
{
  ui->plainTextEdit->setReadOnly(b);
}     

/*! \brief sets prompt_start and prompt_end if isPrompt
  Appending to the mini term is only done for display so it 
  turns off the 'edit_mode' variable to differentiate between
  text added to the buffer as user input (edit_mode = true) or 
  display feedback from elsewhere (edit_mode = false).
  NOTE: This actually does what QT calls 'inserts()' text 
  rather than append() which creates new paragraphs and 
  seems to add redundant line feeds.
*/

void lqMiniTerm::append(const char* buf, bool isPrompt)
{
  // when appending, we can't be in edit mode too..
  bool ed_mode = edit_mode;
  edit_mode = false; 
  if(isPrompt)
  {
    prompt_start = position();
    prompt_end = prompt_start+ strlen(buf);
  }  
  moveCursor(QTextCursor::End);
  editor()->insertPlainText(buf);
  edit_mode = ed_mode;
}


/*!
  \brief Allows low level access to the editors functions.
  This function just returns a pointer to the editor.
*/
lqTermEdit* lqMiniTerm::editor()
{
  return ui->plainTextEdit;
}

int lqMiniTerm::blockNumber()
{
  return editor()->textCursor().blockNumber();
}

/*!
  \brief a wrapper for the editor's moveCursor() function.
  These moves are analogous to edit key operations.
*/

bool lqMiniTerm::moveCursor(QTextCursor::MoveOperation op, 
                            QTextCursor::MoveMode mode /*QTextCursor::MoveAnchor*/)
{
  editor()->moveCursor(op, mode);
  return true;
}

bool lqMiniTerm::setCursorPosition(int pos)
{
  // setPosition(pos) and moveCursor(NoMove) together allow moving the 
  // cursor to an absolute offset in two calls.
  setPosition(pos);
  moveCursor(QTextCursor::NoMove);
  return true;  
}

/*!
  \brief Retrieves last line in the text document containing a prompt.
  Uses the screwy qt editor->document->textblock heirarchy to return the text of 
  the last block written.  Hopefully by you. ;-)
*/

const char* lqMiniTerm::getInput()
{
  QTextBlock b = editor()->document()->findBlock(prompt_start+1);
  QTextBlock b2 = editor()->document()->findBlock(position() - 1);
  if(b == b2)
    return b.text().toAscii();
  else
    return "";
}


static void centerWindow(QWidget* window)
{
  
  int dtx,dty,dtw,dth;
  int winx,winy,winw,winh;
  int x, y;
  
  QDesktopWidget* dt = qApp->desktop();
  dt->availableGeometry().getRect(&dtx,&dty,&dtw,&dth);
  window->geometry().getRect(&winx,&winy,&winw,&winh);
  
  x = dtx + dtw/2 - winw/2; // x1 should not always be 0
  y = dty + dth/2 - winh/2; // y1 ""
  
  window->setGeometry(x, y, winw, winh);
  
  // get it up and shown in the right position now.
  qApp->processEvents();
}

void lqMiniTerm::showEvent(QShowEvent *e)
{
  static bool shown;
  if(shown)
    return;
  shown = true;
  centerWindow(this);
}
