// file: lqminiterm.h

#ifndef LQMINITERM_H
#define LQMINITERM_H

#include <QWidget>
#include <QTextCursor>
#include "lqtermedit.h"
#include "ui_lqminiterm.h"

extern "C" {
typedef long (*LoopFunc)(class lqMiniTerm* term);
typedef long (*HandlerFunc)(class lqMiniTerm* term, uint msg, long wparam, void* lparam);
} // extern "C"

// namespace Ui {
//     class lqMiniTerm;
// }

/*!
  re. the lqMiniTerm message enums (msg_on*):
  These message IDs are used by a Windows-like low level case switch that 
  can be customized to handle each msg as required.
  \enum onClose notifies that it's time to clean up, as req'd.
  \enum onNewline when term->edit_mode is true, indicates that getInput()
  can return a string usable as a command, or whatever.
  \enum onCursorPositionChanged is still somewhat of a mystery.  Used to 
  set the readOnly() mode for the display when the cursor is not on the 
  edit line.  Currently copy/paste only works with the mouse, however and
  you may have to manually set the cursor back on the edit line to paste.
  \enum onKeyPress is used internally by lqTermEdit to make sure that the 
  cursor is at the end of a line when the enter key is pressed.  
  KeyPressEvents are also forwarded to the user defined handler function.
  \enum onExternalInput is used to set the text from a button or menu.
  The parameters passed to the handler are generic and fairly intuitive.
  The first two parameters of the handler function are always obj and msgID
  and the second two are equivalent to (long) value and (void*) reference 
  pointer which are named wparam and lparam after the Windows API, and so 
  integers and shorts, (even if unsigned) would be generally be wparams and 
  things like strings and struct pointers would be lparams, each requiring 
  a cast to and from the appropriate types at each end of the call.
  -
  No, it's not typesafe.  Yes, it is generic and it's fast.
  
 */
enum{
  msg_init = 0,               // 0 (obj,0,0,0)
  msg_onClose,                // 1 (obj,1,0,0)
  msg_onNewline,              // 2 (obj,2,0,0)
  msg_onCursorPositionChanged,// 3 (obj,3,int,0)
  msg_onKeyPress,             // 4 (obj,4,0,QKeyPressEvent*)
  msg_onExternalInput         // 5 (obj,5,0,const char*)
};


class lqMiniTerm : public QWidget
{
    Q_OBJECT

    
public:
    explicit lqMiniTerm(LoopFunc loop_func, HandlerFunc handler_func, QWidget *parent = 0);
    ~lqMiniTerm();

    LoopFunc loopFunc;        // what to run in display mode
    HandlerFunc handler;      // generic signal/event handler
    int prompt_start;         // beginning of prompt string (to avoid)
    int prompt_end;           // end of prompt string (to avoid)
    bool edit_mode;           // as input/output = true/false
    
    int run(int timeout = -1);
    bool closing;   // recursion flag
    
    Ui::lqMiniTerm *ui;
    
    // redefs for editor funcs
    // const QString &toPlainText(); worthless
    const char* getInput();
    
    void setReadOnly(bool b);
    
    int position();
    void setPosition(int position);
    bool moveCursor(
                      QTextCursor::MoveOperation op, 
                      QTextCursor::MoveMode mode = QTextCursor::MoveAnchor);
    bool setCursorPosition(int pos);
    
    void append(const char* buf, bool isPrompt = false);
    
    virtual void showEvent(QShowEvent *e);
        
    lqTermEdit* editor();
    int blockNumber();
    
  public slots:
    void init();
    void onClose();
    void onNewline(int argA); // block changed
    void onCursorPositionChanged();
    void onExternalInput(QString argA);
};



#endif // LQMINITERM_H

